import unittest
from mysql.connector.pooling import MySQLConnectionPool
from qwik_tern.connection import get_connection_pool, create_internal_db
from qwik_tern.logger import setup_logger
from qwik_tern.migration import run_default_migration

logger = setup_logger(__name__)
class TestConnection(unittest.TestCase):
    db_config = {
        'host': 'localhost',
        'port': '3306',
        'user': 'remote',
        'password': 'Nimai@123',
        'database': 'ai_service',
        'auth_plugin': 'mysql_native_password'
    }

    def test_get_connection_pool(self):

        logger.info("Starting test_get_connection_pool")
        pool: MySQLConnectionPool = get_connection_pool(self.db_config)
        self.assertIsNotNone(pool)
        logger.info("Ending test_get_connection_pool")

    # def test_create_internal_db(self):
    #     logger.info("Starting test_create_internal_db")
    #     pool = get_connection_pool(self.db_config)
    #     con = pool.get_connection()
    #     cursor = con.cursor()
    #     cursor.execute("DROP TABLE IF EXISTS db_changelog")
    #     con.commit()

    #     # Run the function to create the table
    #     create_internal_db(pool)

    #     # Check if the table was created
    #     cursor.execute("SHOW TABLES LIKE 'db_changelog'")
    #     result = cursor.fetchone()
    #     self.assertIsNotNone(result)

        
    #     status = run_default_migration(migration_filename ="changes.json", cnx_pool = pool)
    #     self.assertTrue(status)
    #     # Clean up
    #     cursor.execute("DROP TABLE IF EXISTS db_changelog")
    #     con.commit()
        
    #     logger.info("Ending test_create_internal_db")

if __name__ == '__main__':
    unittest.main()


import argparse
from qwik_tern.connection import create_internal_db, get_connection_pool
from qwik_tern.logger import setup_logger
from qwik_tern.migration import run_default_migration, run_migration_down

logger = setup_logger(__name__)

# Argument
# 1. change file Name
# 2. migrate up
# 3. migrate down
if __name__ == '__main__':
    logger.info('Hello, World!')

    # Define the database configuration
    db_config = {
        'host': 'localhost',
        'port': '3306',
        'user': 'remote',
        'password': 'Nimai@123',
        'database': 'ai_service',
        'auth_plugin': 'mysql_native_password'
    }

    # Initialize the connection pool
    cnx_pool = get_connection_pool(db_config)
    if not cnx_pool:
        logger.critical("Failed to create connection pool. Exiting.")
        exit(1)

    # Create the internal database
    create_internal_db(cnx_pool)

    parser = argparse.ArgumentParser(description='Process migration scripts.')
    parser.add_argument('--migrate', type=str, required=True, help='The migrate up script')

    args = parser.parse_args()
    status = False
    if args.migrate.lower() == "up":
        logger.info("Running changes for migration up..")
        status = run_default_migration(migration_filename="changes.json", cnx_pool=cnx_pool)
        if status:
            logger.info("Migration completed successfully")
        else:
            logger.critical("Migration failed")
    elif args.migrate.lower() == "down":
        logger.info("Running DOWN migration By One change")
        # Example: Handle rollback by reading and processing a down migration file
        status = run_migration_down(migration_filename="changes.json", cnx_pool=cnx_pool)
        if status:
            logger.info("Migration completed successfully")
        else:
            logger.critical("Migration failed")
    else:
        logger.warning(f"Unknown migration direction, try: python main.py --help")


import string
import hashlib
import pandas as pd
from qwik_tern.connection import MySQLConnectionPool
import sys

from qwik_tern.logger import setup_logger

logger = setup_logger(__name__)

def read_migration_from_file(file_name: string) -> pd.Series:
    try:
        changes : pd.Series = pd.read_json(file_name).changeSet
        return changes
    except Exception as e:
        logger.critical(f"Error: {e}")
        return None

def run_default_migration(migration_filename: string, cnx_pool: MySQLConnectionPool) -> bool:
    changes : pd.Series = read_migration_from_file(migration_filename)
    if changes is None:
        logger.info("No changes found in the migration file.")
        sys.exit("Terminating program due to error reading the migration file.")
    _process_migration(cnx_pool=cnx_pool, changes=changes)
    return True

def run_migration_down(migration_filename: string, cnx_pool: MySQLConnectionPool) -> bool:
    changes = read_migration_from_file(migration_filename)
    if changes is None:
        logger.info("No changes found in the migration file.")
        sys.exit("Terminating program due to error reading the migration file.")
        return False
    return _process_migration_down(cnx_pool, changes)

def _process_migration_down(cnx_pool: MySQLConnectionPool, changes: pd.DataFrame) -> bool:
    try:
        con = cnx_pool.get_connection()
        cursor = con.cursor()
        try:
            cursor.execute("""SELECT change_id FROM db_changelog ORDER BY id DESC LIMIT 1""")
            id = cursor.fetchone()[0]
            if id is None:
                logger.info("No change ID found in the db to migrate down")
                return False
            migration = ""
            for change in changes:
                if change.get("id") == id:
                    migration = change.get("migrateDown")
                    break
            logger.info("Migrating doen to change id ", id)
            logger.info("Query to migrating down: ", migration)
            if migration is None:
                logger.info("No down migration script found for id: %d " % id)
                return False
            cursor.execute(migration)
            cursor.execute("DELETE FROM db_changelog WHERE change_id = %s", (id,))
            logger.info("Migration down completed successfully for id: %d" % id)
            con.commit()
            cursor.close()
            con.close()
            return True
        except Exception as e:
            logger.critical(f"Error: {e}")
            return False
    except Exception as e:
        logger.critical(f"Error: {e}")
        return False

def _process_migration(cnx_pool: MySQLConnectionPool, changes: pd.DataFrame) -> bool:
    ids = changes.apply(lambda x: x["id"])
    duplicate_ids = ids[ids.duplicated()]
    if not duplicate_ids.empty:
        logger.critical("Duplicate IDs found: %s", duplicate_ids.tolist())
        sys.exit("Terminating program due to duplicate IDs.")
    else:
        logger.info("No duplicate IDs found")
    
    try:
        con = cnx_pool.get_connection()
        cursor = con.cursor()
        try:
            cursor.execute("""SELECT * FROM db_changelog""")
            db_data = cursor.fetchall()
            columns = [desc[0] for desc in cursor.description]
            data = pd.DataFrame(db_data, columns=columns)
            
            for change in changes:
                keyFile = change.get("id") + change.get("author")
                if change.get("check") is not None:
                    exists = _check_Query_Exectuion(cnx_pool, change)
                    if exists:
                        logger.warning("Changeset already exists.")
                        query = _get_changelog_insert_query(
                            change.get("id"),
                            change.get("author"),
                            _calculate_checksum(change.get("migrateUp")),
                            change.get("description"),
                            "SKIP"
                        )
                        if not _check_if_entry_exists(change.get("id"), change.get("author"), cnx_pool):
                            _add_entry_to_changelog_table(query, cnx_pool)
                        continue
                for _, row in data.iterrows():
                    keyDb = row["change_id"] + row["author"]
                    if keyFile == keyDb:
                        if _compare_checksum(row["checksum"], change.get("migrateUp")):
                            continue
                        else:
                            logger.critical("Invalid Checksum!!! previous checksum: %s current checksum:%s ", row["checksum"], _calculate_checksum(change.get("migrateUp")))
                            sys.exit("Terminating program due to checksum mismatch.")
                        continue
                _insert_data(cnx_pool, change)
                continue
        finally:
            cursor.close()
            con.close()
    except Exception as e:
        logger.critical(f"Error: {e}")
        return False

def _check_Query_Exectuion(cnx_pool: MySQLConnectionPool, change: pd.DataFrame) -> bool:
    try:
        con = cnx_pool.get_connection()
        cursor = con.cursor()
        try:
            checkQuery: str = change.get("check").strip()
            if not checkQuery.startswith("SELECT"):
                logger.critical("Invalid Check Query!!!")
                sys.exit("Terminating program due to invalid check query. Should Always start with SELECT")
            cursor.execute(change.get("check"))
            result = cursor.fetchone()
            if result[0] > 0:
                logger.warning("Changeset already exists.")
                return True
            else:
                logger.warning("Record not found")
                return False
        except Exception as e:
            logger.critical(f"Error: {e}")
            return False
        finally:
            cursor.close()
            con.close()
    except Exception as e:
        logger.critical(f"Error: {e}")
        return False

def _insert_data(cnx_pool, change):
    try:
        con = cnx_pool.get_connection()
        cursor = con.cursor()
        try:
            cursor.execute("""
                SELECT COUNT(*) FROM db_changelog 
                WHERE change_id = %s AND author = %s
            """, (change.get("id"), change.get("author")))
            
            result = cursor.fetchone()
            if result[0] > 0:
                logger.warning("Record already exists.")
                return False
            
            status = False
            logger.info("Migration Up: %s", change.get("migrateUp"))
            try:
                cursor.execute(change.get("migrateUp"))
                status = True
            except Exception as e:
                status = False
            statusText = "SUCCESS" if status else "FAILED"
            
            query = _get_changelog_insert_query(change.get("id"), change.get("author"), _calculate_checksum(change.get("migrateUp")), change.get("description"), statusText)
            
            cursor.execute(query)
            con.commit()
        except Exception as e:
            logger.critical(f"Error: {e}")
            return False
        finally:
            cursor.close()
            con.close()
        return True
    except Exception as e:
        logger.critical(f"Error: {e}")
        return False

def _get_changelog_insert_query(change_id: string, author: string, checksum: string, description: string, status: string) -> string:
    return f"""
        INSERT INTO db_changelog (change_id, author, checksum, description, status)
        VALUES ('{change_id}', '{author}', '{checksum}', '{description}', '{status}');
    """

def _check_if_entry_exists(change_id: string, author: string, cnx: MySQLConnectionPool) -> bool:
    try:
        con = cnx.get_connection()
        cursor = con.cursor()
        try:
            cursor.execute("""
                SELECT COUNT(*) FROM db_changelog 
                WHERE change_id = %s AND author = %s
            """, (change_id, author))
            result = cursor.fetchone()
            return result[0] > 0
        finally:
            cursor.close()
            con.close()
    except Exception as e:
        logger.critical(f"Error: {e}")
        return False

def _add_entry_to_changelog_table(query: string, cnx: MySQLConnectionPool) -> bool:
    try:
        con = cnx.get_connection()
        cursor = con.cursor()
        try:
            cursor.execute(query)
            con.commit()
            return True
        except Exception as e:
            logger.critical(f"Error: {e}")
            return False
        finally:
            cursor.close()
            con.close()
    except Exception as e:
        logger.critical(f"Error: {e}")
        return False

def _calculate_checksum(changeText: str) -> str:
    sha256 = hashlib.sha256()
    sha256.update(changeText.encode('utf-8'))
    return sha256.hexdigest()

def _compare_checksum(checksum: str, changeText: str) -> bool:
    return checksum == _calculate_checksum(changeText)





from qwik_tern.connection import get_connection_pool


db_config = {
    'host': 'localhost',
    'port': '3306',
    'user': 'remote',
    'password': 'Nimai@123',
    'database': 'ai_service',
    'auth_plugin': 'mysql_native_password'
}


if __name__ == "__main__":
    con = get_connection_pool(db_config)


# logging_config.py

import logging
import colorlog

def setup_logger(name):
    handler = colorlog.StreamHandler()
    handler.setFormatter(colorlog.ColoredFormatter(
    '%(asctime)s: %(filename)-20s:(%(lineno)4d \t[%(log_color)s %(levelname)s%(reset)s]:\t%(message)s',
    log_colors={
        'DEBUG': 'cyan',
        'INFO': 'green',
        'WARNING': 'yellow',
        'ERROR': 'red',
        'CRITICAL': 'bold_red',
    }
))


    logger = colorlog.getLogger(name)
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)
    return logger


# __init__.py

from qwik_tern.connection import get_connection_pool, create_internal_db
from qwik_tern.migration import run_default_migration, run_migration_down


__all__ = [
    "get_connection_pool",
    "create_internal_db",
    "run_default_migration",
    "run_migration_down",
]


import string
from mysql.connector.pooling import MySQLConnectionPool
from typing import Dict, Any
from qwik_tern.logger import setup_logger

logger = setup_logger(__name__)

def get_connection_pool(config: Dict[str, Any]) -> Any:
    logger.info("Connecting to server...")
    if not config:
        logger.error("No database configuration found...")
        return
    db_config = config
    try:
        mysql_pool = MySQLConnectionPool(
            pool_name="mysql_pool",
            pool_size=10,
            **db_config
        )
        logger.info("Connected to server...")
        return mysql_pool
    except Exception as e:
        logger.critical(f"Error: {e}")
        return None

def create_internal_db(cnxPool: MySQLConnectionPool) -> bool:
    logger.info("Creating internal database...")
    cnx = cnxPool.get_connection()
    cursor = cnx.cursor()
    try:
        logger.info("Executing SQL command to create db_changelog table...")
        query : string = """
         CREATE TABLE IF NOT EXISTS `db_changelog` (
                `id` INT AUTO_INCREMENT PRIMARY KEY,
                `change_id` VARCHAR(255) NOT NULL,
                `author` VARCHAR(255) NOT NULL,
                `checksum` VARCHAR(255) NOT NULL,
                `description` TEXT NOT NULL,
                `status` VARCHAR(10),
                `created_at` TIMESTAMP DEFAULT CURRENT_TIMESTAMP
            );
        """
        logger.info(f"Executing query: {query}")
        cursor.execute(query)
        logger.info("SQL command executed successfully.")
        cnx.commit()
        logger.info("Internal database created...")
        return True
    except Exception as e:
        logger.error(f"Error: {e}")
        return False
    finally:
        logger.info("Closing cursor and connection...")
        cursor.close()
        cnx.close()
        logger.info("Cursor and connection closed.")